@@ -833,20 +833,21 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 		}
 
 		{
-			struct sockaddr addr;
+			struct sockaddr_storage addr;
 			int addr_len = sizeof (addr);
 			guint16 port;
+			ircnet *net = serv->network;
 
-			if (!getsockname (serv->sok, &addr, &addr_len))
+			if (!getsockname (serv->sok, (struct sockaddr *)&addr, &addr_len))
 			{
-				if (addr.sa_family == AF_INET)
+				if (addr.ss_family == AF_INET)
 					port = ntohs(((struct sockaddr_in *)&addr)->sin_port);
 				else
 					port = ntohs(((struct sockaddr_in6 *)&addr)->sin6_port);
 
 				g_snprintf (outbuf, sizeof (outbuf), "IDENTD %"G_GUINT16_FORMAT" ", port);
-				if (serv->network && ((ircnet *)serv->network)->user)
-					g_strlcat (outbuf, ((ircnet *)serv->network)->user, sizeof (outbuf));
+				if (net && net->user && !(net->flags & FLAG_USE_GLOBAL))
+					g_strlcat (outbuf, net->user, sizeof (outbuf));
 				else
 					g_strlcat (outbuf, prefs.hex_irc_user_name, sizeof (outbuf));
 
