@@ -117,7 +117,7 @@ static const struct defaultserver def[] =
 	/* Self signed */
 	{0,			"irc.criten.net"},
 
-	{"DALnet", 0},
+	{"DALnet", 0, 0, 0, LOGIN_NICKSERV},
 	/* Self signed */
 	{0,			"us.dal.net"},
 
@@ -189,6 +189,12 @@ static const struct defaultserver def[] =
 	{0,			"irc.gimp.org"},
 	{0,			"irc.gnome.org"},
 
+	{"GlobalGamers", 0},
+#ifdef USE_OPENSSL
+	{0,			"irc.globalgamers.net/+6660"},
+#endif
+	{0,			"irc.globalgamers.net"},
+
 	{"Hashmark",	0},
 	{0,			"irc.hashmark.net"},
 
@@ -357,42 +363,6 @@ static const struct defaultserver def[] =
 
 GSList *network_list = 0;
 
-#if !GLIB_CHECK_VERSION(2,34,0)
-#define g_slist_copy_deep servlist_slist_copy_deep
-/* FIXME copy-paste from gslist.c, should be dumped sometime */
-static GSList*
-servlist_slist_copy_deep (GSList *list, GCopyFunc func, gpointer user_data)
-{
-  GSList *new_list = NULL;
-
-  if (list)
-    {
-      GSList *last;
-
-      new_list = g_slice_new (GSList);
-      if (func)
-        new_list->data = func (list->data, user_data);
-      else
-        new_list->data = list->data;
-      last = new_list;
-      list = list->next;
-      while (list)
-        {
-          last->next = g_slice_new (GSList);
-          last = last->next;
-          if (func)
-            last->data = func (list->data, user_data);
-          else
-            last->data = list->data;
-          list = list->next;
-        }
-      last->next = NULL;
-    }
-
-  return new_list;
-}
-#endif
-
 favchannel *
 servlist_favchan_copy (favchannel *fav)
 {
